/**********************************************************************/
/*  Copyright (c) 2005-2016 by SAS Institute, Inc., Cary, NC, USA.    */
/*  All rights reserved.                                              */
/**********************************************************************/
/*  NAME:        orm_load_link_inst_tbl.sas                           */
/*  VERSION:     6.1                                                  */
/*  DESCRIPTION: Build the link instance table for reporting use.     */
/*  PARAMETERS:  bo_name = business object type name used to filter   */
/*                         the list of link instances                 */
/*               indsn = primary business object table that will be   */
/*                       joined to the 360 links                      */
/*               rk_column = key used to join the &indsn dataset      */
/*                           to the business object link instances    */
/*  PRODUCT:     SAS Enterprise GRC                                   */
/**********************************************************************/

%macro orm_filter_link_inst_tbl(bo_name=,
                                indsn=,
                                rk_column=);

%put NOTE: {START: &sysmacroname};

/* Rebuild the opinfomp.link_instance table */
%orm_load_link_inst_tbl;

/* Subset the filtered_link by &bo_name */
proc sql;
  create table work.filtered_links as
  select *
  from opinfomp.link_instances
  where upcase(business_object_type_nm_1)="%upcase(&bo_name)" or
        upcase(business_object_type_nm_2)="%upcase(&bo_name)";
quit;

/* Stop processing if the work.filtered_links dataset does not exist */
/* or is empty */
%local nobs rc;
%let dsid = %sysfunc(open(work.filtered_links));
%if &dsid %then 
%do;
  %let nobs  = %sysfunc(attrn(&dsid, nlobs));
  %let rc = %sysfunc(close(&dsid));
%end;
%else %let nobs  = 0;
 
%if &nobs > 0 %then 
%do;
 /* Sort the work.filtered_links by business_object_rk_1 first */
  proc sort tagsort data=work.filtered_links;
    by link_type_rk link_instance_rk;
  run;

 /* Transpose the business_object_rk_1 column */
  proc transpose data=work.filtered_links out=work.bo1_links(drop=_NAME_ _LABEL_) let;
    var business_object_rk_1;
    copy link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
    id business_object_rk_nm1;
    by link_type_rk link_instance_rk;
  run;
  quit;

  /* Sort the work.filtered_links by business_object_rk_2 */
  proc sort tagsort data=work.filtered_links;
    by link_type_rk link_instance_rk;
  run;

  /* Transpose the business_object_rk_2 column */
  proc transpose data=work.filtered_links out=work.bo2_links(drop=_NAME_ _LABEL_) let;
    var business_object_rk_2;
    copy link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1  business_object_rk_2 business_object_type_nm_2;
    id business_object_rk_nm2;
    by link_type_rk link_instance_rk;
  run;
  quit;

  /* Sort the work.filtered_links by link_type_rk */
  proc sort tagsort data=work.filtered_links;
    by link_type_rk link_instance_rk;
  run;

 /* Transpose the link_type_rk column */
  proc transpose data=work.filtered_links out=work.botype_links(drop=_NAME_ _LABEL_) let;
    var link_type_nm;
    copy link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
    id link_type_id;
    by link_type_rk link_instance_rk;
  run;
  quit;

  /* Sort the work.filtered_links by link_type_id */
  proc sort tagsort data=work.filtered_links;
    by link_type_id;
  run;

  /* Sort the work.bo2_links to prep for a merge */
  proc sort data=work.bo1_links;
    by link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
  run;

  /* Sort the work.bo2_links to prep for a merge */
  proc sort tagsort data=work.bo2_links;
    by link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
  run;

  /* Sort the work.botype_links to prep for a merge */
  proc sort tagsort data=work.botype_links;
    by link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
  run;

  /* Update the bo1 and bo2 tables */
  data work.bo12_links;
    update work.bo1_links work.bo2_links;
     by link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
  run;

  /* Merge in the bo1 & bo2 links with the botypes */
  data work.bo_links;
    merge bo12_links botype_links;
	by link_type_rk link_instance_rk business_object_rk_1 business_object_type_nm_1 business_object_rk_2 business_object_type_nm_2;
  run;

  /* Get a list of all link_type_ids, link_type_nms and related rk columns */
  /* in order to build a data structure to merge with the &indsn table */
  proc sql noprint; 
    select distinct(link_type_id) into :vars separated by " "
    from filtered_links
    order by link_type_id;

    select distinct "'"|| trim(link_type_nm) ||"'" into :vars_val separated by "*" 
    from filtered_links
    order by link_type_id;

    select distinct substr(substr(name,5),1,length(substr(name,5))-3) into : rk_keys separated by '*'
    from sashelp.vcolumn
    where libname='WORK' and 
          memname='BO_LINKS' and
          substr(name,1,4)='GLB_' and
          name ne "GLB_%sysfunc(translate(%upcase(&rk_column),'_',''))";
  quit;

  %let i=1;
  %let j=1;
  %let var_nm=%qscan(&vars,&i,' ');
  %let var_val=%qscan(%bquote(&vars_val),&i,%str(*));
  %let rk_key_nm=%scan(&rk_keys,&j,'*');

  /* Join the &indsn, work.bo_links and link_types */
  proc sql;
    create table work.all_pbo_rels as
    select distinct PBO.&rk_column,

     %do %while (&rk_key_nm ne );
        GLB_&rk_key_nm._Rk
        label="%sysfunc(sasmsg(sashelp.ormonrpt,Linked, NOQUOTE)) %sysfunc(propcase(%sysfunc(translate(&rk_key_nm,' ','_')))) %sysfunc(sasmsg(sashelp.ormonrpt,Global_Rk, NOQUOTE))",

      strip(put(GLB_&rk_key_nm._rk,8.)) as GLB_&rk_key_nm._key
        label="%sysfunc(sasmsg(sashelp.ormonrpt,Linked, NOQUOTE)) %sysfunc(propcase(%sysfunc(translate(&rk_key_nm,' ','_')))) %sysfunc(sasmsg(sashelp.ormonrpt,Global_Key, NOQUOTE))",
        %let j=%eval(&j+1);
        %let rk_key_nm=%scan(&rk_keys,&j,'*');
    
     %end; 

      %do %while (&var_nm ne );
        bolnktyp.&var_nm
        label="&var_val"
        %let i=%eval(&i+1);
        %let var_nm=%qscan(&vars,&i,' ');
        %let var_val=%qscan(%bquote(&vars_val),&i,%str(*));
   
        %if &var_nm ne %str() %then %str(,);
      %end; 
   from &indsn PBO
   left join bo_links bolnktyp
   on (PBO.&rk_column=bolnktyp.business_object_rk_1 and
       upcase(bolnktyp.business_object_type_nm_1)="%upcase(&bo_name)") OR
      (PBO.&rk_column=bolnktyp.business_object_rk_2 and
       upcase(bolnktyp.business_object_type_nm_2)="%upcase(&bo_name)");

   create table &bo_name._lnks as
   select a.*, b.*
   from &indsn a
   left join all_pbo_rels b
   on a.&rk_column=b.&rk_column;
   
   /* Initialize the &bo_name._count variable to count just the first.&bo_name._rk */
   create table &bo_name._rels as
   select *, monotonic() as rownum, min(calculated rownum) as min,
   case when calculated rownum=calculated min then 1 else 0 end as &bo_name._count length=8 label="%sysfunc(sasmsg(sashelp.ormonrpt,&bo_name._Count, NOQUOTE))"
   from &bo_name._lnks
   group by &rk_column
   order by rownum;
  quit;
  
  /* Drop the rownum and min variables since they are no longer needed */
  proc sql noprint;
    alter table &bo_name._rels 
	drop column rownum, min;
  quit;
%end;

%put NOTE: {END: &sysmacroname};

%mend orm_filter_link_inst_tbl;
