/***************************************************************************/
/*  Copyright (c) 2005-2014 by SAS Institute, Inc., Cary, NC, USA.         */
/*  All rights reserved.                                                   */
/***************************************************************************/
/*  NAME:        orm_load_lasr_tbls.sas                                    */
/*  VERSION:     6.1                                                       */
/*  DESCRIPTION: Loads tables into LASR and applies RLS permissions.       */
/*  PARAMETERS:  INTBLNM - business object dataset used to load into LASR  */
/*               FEATURE - business object feature name                    */
/*               LASRENVSET - lasr environment setup: N (default) , Y      */
/*  PRODUCT:     SAS Enterprise GRC                                        */
/***************************************************************************/

%macro orm_load_lasr_tbls(intblnm=,
                          feature=,
                          lasrEnvSet=Y);

%put NOTE: {START: &sysmacroname}; 

/* Load &intblnm into LASR (Yes or No) */
%local load_lasr;

%orm_get_app_property_value(application=&orm_application_name, property=grc.loadLASRTbls,
                            output_property_value=orm_load_lasr, output_status=orm_status);

%let load_lasr=%trim(%nrstr(&orm_load_lasr));

%if %upcase(&load_lasr) = TRUE %then
%do;
  %if %upcase(&lasrEnvSet) = Y %then
  %do;
    /* Setup the LASR environment variables */
    %orm_get_lasr_env;
  %end;  
    
  /* Load the OPCNTL features and dimension tables. */
  %orm_load_feature_dims;

  /* Build the permission table used for data filtering */
  %orm_create_security_denorm(srclib=&src_lib,
                              dstlib=work);

  /* LASR permission filter macros */
  %orm_lasr_perm_filter; 

  /* Subset the user permissions to just the &intblnm business object. */
  proc sql;
    create table work.users as
    select distinct user_id
    from work.sm_security_acl
    where ((upcase(business_object_nm)="%upcase(&intblnm)" or business_object_nm=''
          or upcase(business_object_nm)="%upcase(&feature)") and
          upcase(active_flg)='Y');
  quit;
  
  /* Check to see if the &intblnm._lasr table exist and drop it */
  %if %sysfunc(exist(grclasr.&intblnm._lasr)) %then
  %do;
    proc datasets library=grclasr nolist;
      delete &intblnm._lasr;
    quit;
        
    /* delete the LASR table in metadata */
    proc metalib;
      omr(library="&egrc_meta_lib");  
      select("&intblnm._lasr"); 
      update_rule=(delete);
    run;
  %end;
  %else
  %do;
    %put %sysfunc(sasmsg(sashelp.ormonrpt, lasr_tbl_not_found, NOQUOTE));
  %end;

  /* Remove the formats from the work.&intblnm table before loading into LASR */
  %orm_unformat_lasr_tbls;
    
  /* Load the &intblnm into LASR */                       
  data grclasr.&intblnm._lasr;
    set work.&intblnm;
  run;

  /* register the LASR table in metadata */
  proc metalib;
    omr(library="&egrc_meta_lib");  
    select("&intblnm._lasr"); 
  run; 

  /* Get macro variable bit flags. */
  %mdseccon();

  /* Set metadata permissions on the &intblnm */
  data _null_;
    format tc $20.;
    length uri user $256 condition wclause $32767;
    tc = '';
    uri='';
    n=1;

    /* Get the &intblnm._LASR PhysicalTable object in metadata. */
    rc=metadata_getnobj("omsobj:PhysicalTable?@Name='&intblnm._lasr'",n,uri);

    /* Start transaction on object created above using the URI. */
    rc=METASEC_BEGTRAN(uri,0,tc); 
    if (rc < 0 ) then 
    do;
      sysmsg = sysmsg();
      put sysmsg;
    end;

    /* Loop through the EGRC users w/ access to the business object to set permission conditions */
    do until (lastrec);
      set work.users end=lastrec;
      user=strip(user_id);

      call execute('%get_user_sec_isUnder_constraints(feature=' || "&feature" || ', user_id=' || user || ', security_acl_lib=work' || ')');
        
      condition=symget('acc');
      if (strip(condition)='0' or strip(condition)='1' or strip(condition)='') then wclause="";
      else wclause='(' || strip(condition) || ')';
        
      /* Give users ReadMetadata permissions on the &intblnm._lasr table */
      rc = METASEC_SETAUTH(tc, "","Person", user,
                          "Grant", "ReadMetadata", wclause,0);
      if (rc < 0 ) then 
      do;
        sysmsg = sysmsg();
        put sysmsg;
      end;    

      /* Give users Read permissions on the &intblnm._lasr table */
      rc = METASEC_SETAUTH(tc, "","Person", user,
                           "Grant", "Read", wclause,0);
      if (rc < 0 ) then 
      do;
        sysmsg = sysmsg();
        put sysmsg;
      end;     
    end;
   
    /* Deny the Enterprise GRC Users Group ReadMetadata permissions on the &intblnm._lasr table */
    /* so that only users w/ access to &intblnm._lasr can see the table */ 
    rc = METASEC_SETAUTH(tc, "","IdentityGroup", "Enterprise GRC Users",
                         "Deny", "ReadMetadata", "",0);
    if (rc < 0 ) then 
    do;
      sysmsg = sysmsg();
      put sysmsg;
    end; 
    
    /* End the transaction and commit any changes made to */
    /* the transaction since it was started.              */
    rc=METASEC_ENDTRAN(uri,tc, &_SECAD_COMMIT_TC ); 
    if (rc < 0 ) then
    do;
      sysmsg = sysmsg();
      put sysmsg;
    end;
  run;
  
  /* Check to see if the oplasr.&intblnm._lasr table exist and drop it */
  %if %sysfunc(exist(oplasr.&intblnm._lasr)) %then
  %do;
    proc datasets library=oplasr nolist;
      delete &intblnm._lasr;
    quit;
   
    /* Put a copy of the grclasr.&intblnm._lasr table in oplasr */
    /* in order to update our backing store to support reload. */
    data oplasr.&intblnm._lasr;
      set grclasr.&intblnm._lasr;
    run;
  %end;
  %else
  %do;
    /* Put a copy of the grclasr.&intblnm._lasr table in oplasr */
    /* in order to update our backing store to support reload. */
    data oplasr.&intblnm._lasr;
      set grclasr.&intblnm._lasr;
    run;
  %end;
%end;

%put NOTE: {END: &sysmacroname}; 
 
%mend orm_load_lasr_tbls;
